@@ -1,11 +1,13 @@
 import io
 import json
+from snappi_trex.validation import Validation
 
 class Capture(object):
 
     def __init__(self, trexclient):
         self._client = trexclient
         self._captures = {}
+        self._port_ids = []
 
     def set_capture(self, payload, port_ids):
         self._state = payload
@@ -14,27 +16,25 @@ class Capture(object):
         ports = list(range(len(port_ids)))
         if cs['port_names'] is not None:
             ports = []
-            for f_name in cs['port_names']:
-                ports.append(port_ids.index(f_name))
+            for p_name in cs['port_names']:
+                ports.append(port_ids.index(p_name))
 
         if cs['state'] == 'start':
             for p in ports:
-                self._captures[p] = self._client.start_capture(rx_ports = [p])
+                if p not in self._captures:
+                    self._captures[p] = self._client.start_capture(rx_ports = [p])
         elif cs['state'] == 'stop':
-            for cap in self._captures.values():
-                self._client.stop_capture(cap['id'])
-            self._captures = {}
+            for p in ports:
+                if p in self._captures:
+                    self._client.stop_capture(self._captures[p]['id'])
+                    self._captures.pop(p)
 
     def get_capture(self, request):
         port_idx = self._port_ids.index(request.port_name)
-        pkt_list = []
-        self._client.fetch_capture_packets(self._captures[port_idx]['id'], pkt_list)
         res = io.BytesIO()
-        for pkt in pkt_list:
-            res.write(pkt['binary'])
-        #     if binary is None:
-        #         binary = pkt['binary']
-        #     else:
-        #         binary += pkt['binary']
-        # res = io.BytesIO(binary)
+        if port_idx in self._captures:
+            pkt_list = []
+            self._client.fetch_capture_packets(self._captures[port_idx]['id'], pkt_list)
+            for pkt in pkt_list:
+                res.write(pkt['binary'])
         return res
\ No newline at end of file
