@@ -202,6 +202,6 @@ class SetConfig:
                     layers_with_len[layer] += 1
 
         # Fill the rest of the packet with x's
-        pad = max(0, max_pkt_size - len(pkt_base)) * 'x'
+        pad = max(0, max_pkt_size - len(pkt_base)) * '\0'
 
         return vm_cmds, pad
