@@ -32,7 +32,7 @@ class Api(snappi.Api):
         )
         self._c = STLClient()
         self._port_ids = []
-        self._transmit_state = 'stop'
+        self._transmit_states = {}
         self._capture = Capture(self._c)
 
         try:
@@ -52,9 +52,9 @@ class Api(snappi.Api):
 
     # Maps port names used in Snappi to port index for T-Rex
     def _loadPorts(self):
-        if 'ports' in self._cfg:
-            i = 0
-            for p in self._cfg['ports']:
+        cfg_obj = json.loads(self._config.serialize())
+        if 'ports' in cfg_obj:
+            for p in cfg_obj['ports']:
                 self._port_ids.append(p['name'])
 
 
@@ -65,15 +65,14 @@ class Api(snappi.Api):
         res = {'warnings': []}
 
         # print(config.serialize())
-        self._cfg = json.loads(config.serialize())
+        self._config = config
+        cfg_obj = json.loads(self._config.serialize())
         self._loadPorts()
 
         try:
-            # prepare our ports
             self._c.reset(ports = list(range(len(self._port_ids))))
-
             # for each Snappi flow, construct the equivalent T-Rex stream
-            for f in self._cfg["flows"]:
+            for f in cfg_obj["flows"]:
                 # Configure variable manager commands
                 vmCmds = []
 
@@ -125,24 +124,52 @@ class Api(snappi.Api):
         """Set the transmit state of flows
         """
         ts = json.loads(payload.serialize())
+        cfg_obj = json.loads(self._config.serialize())
         ports = list(range(len(self._port_ids)))
-        if ts['flow_names'] is not None:
+
+        if payload.flow_names is not None:
+            # Get all transmit ports names from each flow
+            port_names = []
+            for f in cfg_obj['flows']:
+                if f['name'] in ts['flow_names']:
+                    port_names.append(f['tx_rx']['port']['tx_name'])
+
+            # Create port ID list from port names
             ports = []
-            for f_name in ts['flow_names']:
-                ports.append(self._flow_ids.index(f_name))
+            for p_name in port_names:
+                p_idx = self._port_ids.index(p_name)
+                if p_idx not in ports:
+                    ports.append(p_idx)
+
         try:
-            if ts['state'] == 'start':
-                if self._transmit_state == 'stop':
-                    self._c.clear_stats()
-                    self._c.start(ports = ports)
-                elif self._transmit_state == 'pause':
-                    self._c.resume(ports = ports)
-            elif ts['state'] == 'stop':
-                self._c.stop(ports = ports)
-            elif ts['state'] == 'pause':
-                self._c.pause(ports = ports)
-            
-            self._transmit_state = ts['state']
+            ports_start = []
+            ports_stop = []
+            ports_resume = []
+            ports_pause = []
+            # Sort each port into start, stop, resume, or pause based on current state
+            for p in ports:
+                if ts['state'] == 'start':
+                    if p not in self._transmit_states or self._transmit_states[p] == 'stop':
+                        ports_start.append(p)
+                    elif self._transmit_states[p] == 'pause':
+                        ports_resume.append(p)
+                elif ts['state'] == 'stop':
+                    ports_stop.append(p)
+                elif ts['state'] == 'pause':
+                    ports_pause.append(p)
+
+                self._transmit_states[p] = ts['state']
+
+            if len(ports_start) > 0:
+                self._c.clear_stats(ports = ports_start)
+                self._c.start(ports = ports_start)
+            if len(ports_stop) > 0:
+                self._c.stop(ports = ports_stop)
+            if len(ports_resume) > 0:
+                self._c.resume(ports = ports_resume)
+            if len(ports_pause) > 0:
+                self._c.pause(ports = ports_pause)
+
             self._c.wait_on_traffic(ports = ports)
         except STLError as e:
             self._c.disconnect()
@@ -153,6 +180,7 @@ class Api(snappi.Api):
         """Starts capture on all ports that have capture enabled.
         """
         try:
+            Validation.validate_capture(payload, self._port_ids)
             self._capture.set_capture(payload, self._port_ids)
         except STLError as e:
             self._c.disconnect()
@@ -162,6 +190,7 @@ class Api(snappi.Api):
     def get_capture(self, request):
         """Gets capture file and returns it as a byte stream
         """
+        Validation.validate_capture_request(request, self._port_ids)
         return self._capture.get_capture(request)
         
 
